@@ -22,6 +22,7 @@ class PrettyErrorsConfig():
         self.seperator_character    = '-'
         self.line_number_first      = False
         self.top_first              = False
+        self.always_display_bottom  = True
         self.stack_depth            = 0
         self.exception_above        = False
         self.exception_below        = True
@@ -31,16 +32,23 @@ class PrettyErrorsConfig():
         self.lines_after            = 0
         self.display_locals         = False
         self.display_trace_locals   = False
+        self.truncate_locals        = True
+        self.truncate_code          = False
+        self.line_prefix            = ''
+        self.code_prefix            = ''
         self.header_color           = '\033[1;30m'
         self.timestamp_color        = '\033[1;30m'
         self.line_color             = '\033[1;38m'
         self.code_color             = '\033[1;30m'
+        self.line_prefix_color      = '\033[1;38m'
+        self.code_prefix_color      = '\033[1;30m'
         self.filename_color         = '\033[1;36m'
         self.line_number_color      = '\033[1;32m'
         self.function_color         = '\033[1;34m'
         self.link_color             = '\033[1;30m'
         self.local_name_color       = '\033[1;35m'
         self.local_value_color      = '\033[m'
+        self.local_len_color        = '\033[1;30m'
         self.exception_color        = '\033[1;31m'
         self.exception_arg_color    = '\033[1;33m'
         self.prefix                 = None
@@ -52,7 +60,7 @@ class PrettyErrorsConfig():
 
 
     def configure(self, **kwargs):
-        """Used to configure settings governing how exceptions are displayed."""
+        """Configure settings governing how exceptions are displayed."""
         for setting in kwargs:
             if kwargs[setting] is not None: setattr(self, setting, kwargs[setting])
 
@@ -60,67 +68,107 @@ class PrettyErrorsConfig():
 config = PrettyErrorsConfig()
 
 
-def configure(line_length = None, filename_display = None, full_line_newline = None,
-              display_timestamp = None, display_link = None, seperator_character = None,
-              header_color = None, line_color = None, code_color = None,
-              timestamp_color = None, filename_color = None, line_number_color = None,
-              function_color = None, link_color = None,
-              exception_color = None, exception_arg_color = None,
-              prefix = None, infix = None, postfix = None,
-              trace_lines_before = None, trace_lines_after = None,
-              lines_before = None, lines_after = None,
-              display_locals = None, display_trace_locals = None,
-              local_name_color = None, local_value_color = None,
-              top_first = None, stack_depth = None, line_number_first = None,
-              exception_above = None, exception_below = None, reset_stdout = None):
-    """Used to configure settings governing how exceptions are displayed."""
+def configure(
+        always_display_bottom = None,
+        code_color = None,
+        code_prefix = None,
+        code_prefix_color = None,
+        display_link = None,
+        display_locals = None,
+        display_timestamp = None,
+        display_trace_locals = None,
+        exception_above = None,
+        exception_arg_color = None,
+        exception_below = None,
+        exception_color = None,
+        filename_color = None,
+        filename_display = None,
+        full_line_newline = None,
+        function_color = None,
+        header_color = None,
+        infix = None,
+        line_color = None,
+        line_length = None,
+        line_number_color = None,
+        line_number_first = None,
+        line_prefix = None,
+        line_prefix_color = None,
+        lines_after = None,
+        lines_before = None,
+        link_color = None,
+        local_len_color = None,
+        local_name_color = None,
+        local_value_color = None,
+        postfix = None,
+        prefix = None,
+        reset_stdout = None,
+        seperator_character = None,
+        stack_depth = None,
+        timestamp_color = None,
+        top_first = None,
+        trace_lines_after = None,
+        trace_lines_before = None,
+        truncate_code = None,
+        truncate_locals = None
+        ):
+    """Configure settings governing how exceptions are displayed."""
     config.configure(
-        line_length            = line_length,
+        always_display_bottom  = always_display_bottom,
+        code_color             = code_color,
+        code_prefix            = code_prefix,
+        code_prefix_color      = code_prefix_color,
+        display_link           = display_link,
+        display_locals         = display_locals,
+        display_timestamp      = display_timestamp,
+        display_trace_locals   = display_trace_locals,
+        exception_above        = exception_above,
+        exception_arg_color    = exception_arg_color,
+        exception_below        = exception_below,
+        exception_color        = exception_color,
+        filename_color         = filename_color,
         filename_display       = filename_display,
         full_line_newline      = full_line_newline,
-        display_timestamp      = display_timestamp,
-        display_link           = display_link,
-        seperator_character    = seperator_character,
-        header_color           = header_color,
-        line_color             = line_color,
-        code_color             = code_color,
-        timestamp_color        = timestamp_color,
-        filename_color         = filename_color,
-        line_number_color      = line_number_color,
         function_color         = function_color,
-        link_color             = link_color,
-        exception_color        = exception_color,
-        exception_arg_color    = exception_arg_color,
-        prefix                 = prefix,
+        header_color           = header_color,
         infix                  = infix,
-        postfix                = postfix,
-        exception_above        = exception_above,
-        exception_below        = exception_below,
-        trace_lines_before     = trace_lines_before,
-        trace_lines_after      = trace_lines_after,
-        lines_before           = lines_before,
+        line_color             = line_color,
+        line_length            = line_length,
+        line_number_color      = line_number_color,
+        line_number_first      = line_number_first,
+        line_prefix            = line_prefix,
+        line_prefix_color      = line_prefix_color,
         lines_after            = lines_after,
-        display_locals         = display_locals,
-        display_trace_locals   = display_trace_locals,
+        lines_before           = lines_before,
+        link_color             = link_color,
+        local_len_color        = local_len_color,
         local_name_color       = local_name_color,
         local_value_color      = local_value_color,
-        top_first              = top_first,
-        line_number_first      = line_number_first,
+        postfix                = postfix,
+        prefix                 = prefix,
+        reset_stdout           = reset_stdout,
+        seperator_character    = seperator_character,
         stack_depth            = stack_depth,
-        reset_stdout           = reset_stdout
+        timestamp_color        = timestamp_color,
+        top_first              = top_first,
+        trace_lines_after      = trace_lines_after,
+        trace_lines_before     = trace_lines_before,
+        truncate_code          = truncate_code,
+        truncate_locals        = truncate_locals
     )
 
 
 def whitelist(*paths):
     """If the whitelist has any entries, then only files which begin with
-    one of its entries will be included in the stack trace"""
+    one of its entries will be included in the stack trace.
+    """
     for path in paths:
         config.whitelist_paths.append(os.path.normpath(path).lower())
 
 
 def blacklist(*paths):
     """Files which begin with a path on the blacklist will not be
-    included in the stack trace."""
+    included in the stack trace.
+    """
     for path in paths:
         config.blacklist_paths.append(os.path.normpath(path).lower())
 
@@ -169,7 +217,7 @@ def excepthook(exception_type, exception_value, traceback):
         else:
             seperator = line_length * config.seperator_character
         output_text('\n')
-        output_text([config.header_color, seperator], newline = True)
+        output_text([config.header_color, seperator], newline=True)
 
 
     def write_location(path, line, function):
@@ -177,8 +225,8 @@ def excepthook(exception_type, exception_value, traceback):
         output_text('\n')
         if config.filename_display == FILENAME_FULL:
             filename = ""
-            output_text([config.filename_color, path], newline = True)
-            output_text([config.line_number_color, line_number, config.function_color, function], newline = True)
+            output_text([config.filename_color, path], newline=True)
+            output_text([config.line_number_color, line_number, config.function_color, function], newline=True)
         else:
             if config.filename_display == FILENAME_EXTENDED:
                 line_length = get_line_length()
@@ -192,15 +240,15 @@ def excepthook(exception_type, exception_value, traceback):
                     config.line_number_color, line_number,
                     config.function_color,    function + ' ',
                     config.filename_color,    filename
-                ], newline = True)
+                ], newline=True)
             else:
                 output_text([
                     config.filename_color,    filename + ' ',
                     config.line_number_color, line_number,
                     config.function_color,    function
-                ], newline = True)
+                ], newline=True)
         if config.display_link:
-            output_text([config.link_color, '"%s", line %s' % (path, line)], newline = True)
+            output_text([config.link_color, '"%s", line %s' % (path, line)], newline=True)
 
 
     def write_code(filepath, line, module_globals, is_final):
@@ -223,6 +271,7 @@ def excepthook(exception_type, exception_value, traceback):
 
         min_lead = 9999
         for line in lines:
+            if line.strip() == '': continue
             c = 0
             while c < len(line) and line[c] in (' ', '\t'):
                 c += 1
@@ -230,12 +279,24 @@ def excepthook(exception_type, exception_value, traceback):
         if min_lead > 0:
             lines = [line[min_lead:] for line in lines]
 
+        if config.truncate_code:
+            line_length = get_line_length()
 
         for i, line in enumerate(lines):
             if i == target_line:
-                output_text([config.line_color, line], newline = True)
+                prefix_color = config.line_prefix_color
+                prefix       = config.line_prefix
+                color        = config.line_color
             else:
-                output_text([config.code_color, line], newline = True)
+                prefix_color = config.code_prefix_color
+                prefix       = config.code_prefix
+                color        = config.code_color
+            if config.truncate_code and len(line) + len(prefix) > line_length:
+                line = line[:line_length - (len(prefix) + 3)] + '...'
+            if prefix != '':
+                output_text([prefix_color, prefix, reset_color, color, line], newline=True)
+            else:
+                output_text([color, line], newline=True)
 
         return '\n'.join(lines)
 
@@ -252,9 +313,9 @@ def excepthook(exception_type, exception_value, traceback):
             output_text([
                 config.exception_color, exception_name(exception_type), ':\n',
                 config.exception_arg_color, '\n'.join((str(x) for x in exception_value.args))
-            ], newline = True)
+            ], newline=True)
         else:
-            output_text([config.exception_color, exception_name(exception_type)], newline = True)
+            output_text([config.exception_color, exception_name(exception_type)], newline=True)
 
 
     write_header()
@@ -263,21 +324,21 @@ def excepthook(exception_type, exception_value, traceback):
         sys.stderr.write(config.prefix)
 
     if config.exception_above:
-        output_text('', newline = True)
+        output_text('', newline=True)
         write_exception(exception_type, exception_value)
 
     tracebacks = []
     while traceback != None:
         path = os.path.normpath(traceback.tb_frame.f_code.co_filename).lower()
-        if config.whitelist_paths and traceback.tb_next:
-            for white in config.whitelist_paths:
-                if path.startswith(white): break
-            else:
-                traceback = traceback.tb_next
-                continue
-        if traceback.tb_next == None:
+        if traceback.tb_next == None or (config.always_display_bottom and tracebacks == []):
             tracebacks.append(traceback)
         else:
+            if config.whitelist_paths:
+                for white in config.whitelist_paths:
+                    if path.startswith(white): break
+                else:
+                    traceback = traceback.tb_next
+                    continue
             for black in config.blacklist_paths:
                 if path.startswith(black): break
             else:
@@ -287,11 +348,17 @@ def excepthook(exception_type, exception_value, traceback):
     if config.top_first:
         tracebacks.reverse()
         if config.stack_depth > 0:
-            tracebacks = tracebacks[:config.stack_depth]
+            if config.always_display_bottom and len(tracebacks) > 1:
+                tracebacks = tracebacks[:config.stack_depth] + tracebacks[-1:]
+            else:
+                tracebacks = tracebacks[:config.stack_depth]
         final = 0
     else:
         if config.stack_depth > 0:
-            tracebacks = tracebacks[-config.stack_depth:]
+            if config.always_display_bottom and len(tracebacks) > 1:
+                tracebacks = tracebacks[:1] + tracebacks[-config.stack_depth:]
+            else:
+                tracebacks = tracebacks[-config.stack_depth:]
         final = len(tracebacks) - 1
 
     for count, traceback in enumerate(tracebacks):
@@ -307,14 +374,24 @@ def excepthook(exception_type, exception_value, traceback):
             local_variables = [(code_string.find(x), x) for x in frame.f_locals]
             local_variables.sort()
             local_variables = [x[1] for x in local_variables if x[0] >= 0]
-            for local in local_variables:
-                output_text([
-                    config.local_name_color, local, ': ',
-                    config.local_value_color, frame.f_locals[local],
-                ], newline = True)
+            if local_variables:
+                output_text('', newline=True)
+                spacer = ': '
+                len_spacer = '... '
+                line_length = get_line_length()
+                for local in local_variables:
+                    value = str(frame.f_locals[local])
+                    output = [config.local_name_color, local, spacer, config.local_value_color]
+                    if config.truncate_locals and len(local) + len(spacer) + len(value) > line_length:
+                        length = '[' + str(len(value)) + ']'
+                        value = value[:line_length - (len(local) + len(spacer) + len(len_spacer) + len(length))]
+                        output += [value, len_spacer, config.local_len_color, length]
+                    else:
+                        output += [value]
+                    output_text(output, newline=True)
 
     if config.exception_below:
-        output_text('', newline = True)
+        output_text('', newline=True)
         write_exception(exception_type, exception_value)
 
     if config.postfix != None:
@@ -325,10 +402,72 @@ sys.excepthook = excepthook
 
 
 
+def install(skip_query = False):
+    """Install pretty_errors in your sitecustomize.py file, which means
+    it will be present whenever you run a python file.
+    """
+    import click, re, site
+    sitecustomize_path = os.path.join(site.USER_SITE, 'sitecustomize.py')
+    try:
+        sitecustomize = ''.join((x for x in open(sitecustomize_path)))
+    except IOError:
+        sitecustomize = ''
+    if re.search(r'^\s*import\s+\bpretty_errors\b', sitecustomize, re.MULTILINE):
+        print('\npretty_errors already present in:\n  ' + sitecustomize_path +
+              '\nEdit it to set config options.')
+    else:
+        if not skip_query and not click.confirm(
+                'Add pretty_errors to your sitecustomize.py file?  '
+                'If so it will be used whenever you run any python file.',
+                default=True):
+            return
+        output = []
+        output.append('''
+
+# pretty-errors package to make exception reports legible.
+import pretty_errors
+"""
+pretty_errors.configure(''')
+        options = []
+        colors = []
+        parameters = []
+        max_length = 0
+        for option in dir(config):
+            if len(option) > max_length:
+                max_length = len(option)
+            if (option not in ('configure', 'whitelist_paths', 'blacklist_paths') and
+                    not option.startswith('_')):
+                if option.endswith('_color'):
+                    colors.append(option)
+                else:
+                    options.append(option)
+        for option in sorted(options):
+            if option == 'filename_display':
+                parameters.append('    ' + option.ljust(max_length) + ' = pretty_errors.FILENAME_COMPACT,  # FILENAME_EXTENDED | FILENAME_FULL')
+            else:
+                parameters.append('    ' + option.ljust(max_length) + ' = ' + repr(getattr(config, option)))
+        for option in sorted(colors):
+            parameters.append('    ' + option.ljust(max_length) + ' = ' + repr(getattr(config, option)))
+
+        output.append(',\n'.join(parameters))
+        output.append(')')
+        output.append('"""\n')
+
+        try:
+            os.makedirs(site.USER_SITE)
+        except Exception:
+            pass
+        try:
+            out = open(sitecustomize_path, 'a')
+            out.write('\n'.join(output))
+            out.close()
+        except Exception:
+            print('\nFailed to write to:\n' + sitecustomize_path)
+        else:
+            print('\npretty_errors added to:\n  ' + sitecustomize_path +
+                  '\nEdit it to set config options.')
+
+
+
 if __name__ == "__main__":
-    configure(lines_before = 2, lines_after = 2, display_locals = True)
-    #blacklist("c:/")
-    alpha = "A"
-    hello = "Hi"
-    raise KeyError("foo", 1)
-        #test
+    install(skip_query=('-y' in sys.argv))
